#ifndef GUI_H
#define GUI_H

#ifdef DOGUI

#include "CC.h"
#include <SDL_mixer.h>

#define X_OFFSET 13
#define Y_OFFSET 11
#define X_SPACE 10.8
#define Y_SPACE 18.5
#define FONT "-adobe-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*"
#define EDGE 3
#define MARBLE_W 15	/* marble width spacing in gadgets */
#define MARBLE_H 15	/* marble height spacing in gadgets */
#define MIN_X 213	/* minimum for right side boxes */
#define MAX_X 107	/* maximum for leftside boxes */
#define MAX_Y 92	/* maximum for top side boxes */
#define MIN_Y 276	/* minimum for bottom side boxes */
#define ANIM_HEIGHT_V 20	/* jump height for vertical animation */
#define ANIM_HEIGHT_S 16	/* jump height for sideways animation */
#define ANIM_JUMP_TIME 160000	/* time in usecs for each full jump animate */
#define TEST_FRAMES 180		/* number of frames to use to test FPS */

#define RADIAN (3.1415926532/180.0)

/* Button groups */
#define BU_H 0
#define BU_V 1
#define BU_FLAG 0
#define BU_CYCLE 1

#define handleIDCMP(a) JASDL_EventPollLoop()

typedef struct BUTTONS
{
	int type;
	void *data;
	int (*func)();
} Buttons;

typedef struct HOTSPOTS
{
	int x,y,w,h,state;
	Buttons *button;
	struct HOTSPOTS *next;
} Hotspots;

typedef struct {
	int x,y;
} Point;

/* functions for all */
int  startgui(void);
void initmarbles(void);
void cleanExit(int);	/* exit value */
void MoveMarble(int);	/* player */
int  getplayers(void);
//int  handleIDCMP(int);
void getclick(int);
void gameover(int);		/* player +=winner -=loser */


extern JASDL_Font *font;

/* functions only for x */

/* Event functions */
int handleEvents(SDL_Event *e);
int do_coord_out();
int handle_expose();
int do_nothing();
int do_quit();
int do_newgame();
int do_options();
int do_gp_done();

void outlineMoves(int, int);		/* on=1/off=0, player */
void SizeButtons(int, int, Buttons*, int*, int*);
							/* orientation,num_buttons,buttons,width,height */
int DrawButtons(int,int,Buttons*,int,int);
									/* orientation,num_buttons,buttons,sx,sy */
void PutMarble(int,int,int);				/* num,x,y */
//int  AdjustMarbles(Jax*);
void DrawGrid(int, int, int, float, float);		/* num, x, y, dx, dy */
void DrawButtonText(Buttons*, int, int, int, int);	/* button, x, y, w, state */
//void DrawGadget(int, int, int, int, int);		/* lightness(1-3), x,y,w,h */
void DrawGadgetBox(int, int, int, int);			/* x, y, w, h */
void DrawInvGadgetBox(int, int, int, int);		/* x, y, w, h */
void UpdateWin(int);
int XtoC(int);
int YtoR(int);
int RtoY(int);
int CtoX(int);
int text_print(int, int, char*);		/* x,y, str */
int textnl_print(int, int, char*);		/* x,y, str */
int texto_print(int, int, char*);		/* x,y, str */
int textc_print(int, int, int, char*);	/* x,y,w, str */
int textoc_print(int, int, int, char*);	/* x,y,w, str */
int text_height(void);
int text_width(char*);
void free_hotspots(void);
void set_hotspot_state(void*, int);				/* data, state */
int do_hotspots();								/* event calls this somehow... */
void add_hotspot(int, int, int, int, Buttons*);
int clear_hotspots(void);

/* vars for all */

extern BOOL button;

/* vars for only x */

//extern Pixmap winbuf, oldwinbuf, newwinbuf; /* oldwinbuf is a temp holder */

//extern int mouse_x, mouse_y;

extern Hotspots *hotspots, *hotspot;

extern int hints, animation, zoom;
extern char *hint_cyc[3], *anim_cyc[3];//, *zoom_cyc[3];
extern Buttons GO_buttons[],game_buttons[],option_buttons[],GP_buttons[];

//extern Jax_Events jax_events[];
//extern int noexpose;

//extern Pixmap ccboard;
extern SDL_Surface *ccboard,*winbuf,*newwinbuf,*oldwinbuf,*selection;
//extern Pixmap ccboard_mask;
//extern XpmAttributes xpma_ccboard;

//extern Pixmap ccmarble[6];
extern SDL_Surface *ccmarble[6];
//extern Pixmap ccmarble_mask[6];
//extern XpmAttributes xpma_ccmarble[6];

//extern Pixmap ccgadget[3];
extern SDL_Surface *ccgadget[3];
//extern Pixmap ccgadget_mask[3];
//extern XpmAttributes xpma_ccgadget[3];

extern Mix_Music *music[5];
extern Mix_Chunk *sfx[7];
int channels, channel;

void playsound(int sound);

#endif /* DOGUI */

#endif /* GUI_H */
