#include <stdlib.h>
#include <string.h>

#include "jasdl.h"

SDL_Surface *s;
SDL_Event e;
Uint8 needRedraw,done;
#ifdef DOJOY
Uint8 joyEvent;
Sint16 joyY[4],joyX[4];
Uint32 joyButtons[4];
#endif
Uint16 mouseX, mouseY;
Uint8  mouseButtons;
RedrawFunc redraw;
HandleEventFunc handleEvent;

void JASDL_Init()
{
	if(SDL_Init(SDL_INIT_EVERYTHING^SDL_INIT_CDROM)!=0)
		JASDL_Exit("SDL_Init");
#ifdef DOJOY
	memset(joyX,0,sizeof(joyX));
	memset(joyY,0,sizeof(joyY));
	memset(joyButtons,0,sizeof(joyButtons));
	joyEvent=0;
#endif
	redraw=NULL;
	done=0;
	needRedraw=1;
	handleEvent=JASDL_BasicHandleEvent;
	s=NULL;
}

void JASDL_Errorv(char *str, va_list ap)
{
	vfprintf(stderr,str,ap);
	fprintf(stderr,": %s.\n", SDL_GetError());
}

void JASDL_Error(char *str,...)
{
	va_list ap;
	va_start(ap, str);
	JASDL_Errorv(str,ap);
	va_end(ap);
}

void JASDL_Exit(char *str,...)
{
	if(str)
	{
		va_list ap;
		va_start(ap, str);
		JASDL_Errorv(str,ap);
		va_end(ap);
	}
#ifdef DOGUI
	if(s)
		SDL_Quit();
#endif
	exit(1);
}

void JASDL_SafeLockSurface(SDL_Surface *s)
{
	int err=0;
	if(SDL_MUSTLOCK(s))
		do
		{
			if(err)
				SDL_Delay(10);
			if((err=SDL_LockSurface(s))==-1)
				JASDL_Exit("SDL_LockSurface");
		}
		while(err==-2);
}

void JASDL_SafeUnlockSurface(SDL_Surface *s)
{
	if(SDL_MUSTLOCK(s))
		SDL_UnlockSurface(s);
}

void JASDL_EventPollLoop()
{
	done=0;
	while(!done && SDL_PollEvent(&e))
		handleEvent(&e);
	if(!done && needRedraw && redraw)
		redraw();
}

void JASDL_EventWaitLoop()
{
	done=0;
	if(needRedraw && redraw)
		redraw();
	while(!done && SDL_WaitEvent(&e))
	{
		handleEvent(&e);
		if(!done)
			JASDL_EventPollLoop();
	}
}

int JASDL_BasicHandleEvent(SDL_Event *e)
{
#ifdef DOJOY
	joyEvent=0;
#endif
	switch(e->type)
	{
		case SDL_MOUSEMOTION:
			mouseX=e->motion.x;
			mouseY=e->motion.y;
			mouseButtons=e->motion.state;
			break;
		case SDL_MOUSEBUTTONDOWN:
		case SDL_MOUSEBUTTONUP:
			mouseX=e->button.x;
			mouseY=e->button.y;
			mouseButtons=e->button.state;
			break;
#ifdef DOJOY
		case SDL_JOYAXISMOTION:
			if(e->jaxis.which>=4)
				return(0);
			if(e->jaxis.axis==0)
				joyX[e->jaxis.which]=e->jaxis.value;
			if(e->jaxis.axis==1)
				joyY[e->jaxis.which]=e->jaxis.value;
			joyEvent=1;
			break;
		case SDL_JOYBUTTONDOWN:
			if(e->jbutton.which>=4 || e->jbutton.button>=32)
				return(0);
			joyButtons[e->jaxis.which]&=~(1<<e->jbutton.button);
			joyEvent=1;
			break;
		case SDL_JOYBUTTONUP:
			if(e->jbutton.which>=4 || e->jbutton.button>=32)
				return(0);
			joyButtons[e->jaxis.which]|=(1<<e->jbutton.button);
			joyEvent=1;
			break;
#endif
		case SDL_KEYDOWN:
			switch(e->key.keysym.sym)
			{
				case SDLK_ESCAPE:
					done=1;
					break;
				default:
					break;
			}
			break;
		case SDL_QUIT:
			done=1;
			break;
		default:
			//printf("Event %d unhandled\n",e->type);
			return(0);
	}
	return(1);
}

void JASDL_PutPixel(SDL_Surface *surface, int x, int y, Uint32 P)
{
	int bpp=surface->format->BytesPerPixel;
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
	
	JASDL_PutRawPixel(p,P,bpp);
}

Uint32 JASDL_GetPixel(SDL_Surface *surface, int x, int y)
{
	int bpp=surface->format->BytesPerPixel;
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

	return(JASDL_GetRawPixel(p,bpp));
}


Uint8 *JASDL_CreateIconMask(SDL_Surface *surface)
{
	int x,y,i;
	Uint8 rgb,a;
	Uint8 *mask;
	
	i=(surface->w*surface->h)/8;
	mask=(Uint8*)malloc(i);
	memset(mask,0,i);
	i=0;
	for(y=0;y<surface->h;y++)
		for(x=0;x<surface->w;x++)
		{
			mask[i/8]<<=1;
			SDL_GetRGBA(JASDL_GetPixel(surface,x,y),surface->format,&rgb,&rgb,&rgb,&a);
			if(a==0xff)
				mask[i/8]|=1;
			i++;
		}
	return(mask);
}

SDL_Surface *JASDL_DisplayFormatAlpha(SDL_Surface *src)
{
	SDL_Surface *dst;

	if(!src)
		return(NULL);
	SDL_SetAlpha(src,(src->flags&SDL_SRCALPHA)|SDL_RLEACCEL,src->format->alpha);
	dst=SDL_DisplayFormatAlpha(src);
	if(!dst)
		return(src);
	SDL_FreeSurface(src);
	return(dst);
}

SDL_Surface *JASDL_CreateDisplaySurface(Uint32 w, Uint32 h)
{
	return(SDL_CreateRGBSurface(s->flags, w, h, s->format->BitsPerPixel,
			s->format->Rmask, s->format->Gmask, s->format->Bmask, s->format->Amask));
}

SDL_Surface *JASDL_CopySurface(SDL_Surface *src, SDL_Rect *rect)
{
	SDL_Surface *dst;
	int w,h;
	Uint32 alpha;

	if(rect)
	{
		w=rect->w;
		h=rect->h;
	}
	else
	{
		w=src->w;
		h=src->h;
	}
	dst=SDL_CreateRGBSurface(src->flags, w, h, src->format->BitsPerPixel,
			src->format->Rmask, src->format->Gmask, src->format->Bmask, src->format->Amask);
	if(src->format->BitsPerPixel==8)
		SDL_SetColors(dst,src->format->palette->colors,0,src->format->palette->ncolors);
	alpha=src->flags&(SDL_SRCALPHA|SDL_RLEACCEL);
	SDL_SetAlpha(src,0,0);
	SDL_BlitSurface(src,rect,dst,0);
	SDL_SetAlpha(src,alpha,0);
	return(dst);
}

void JASDL_RotateFlip(SDL_Surface *s, JASDL_RotateFlipType rf)
{
	int x,y,bpp=s->format->BytesPerPixel,w=s->pitch;
	Uint32 p,p2;
	Uint8 *ptr,*ptr2;
	
	if(s->h!=s->w)
		return;
	JASDL_SafeLockSurface(s);
#define swap { \
	/*printf("swap %p %p\n",ptr,ptr2);*/ \
	p=JASDL_GetRawPixel(ptr,bpp); \
	p2=JASDL_GetRawPixel(ptr2,bpp); \
	JASDL_PutRawPixel(ptr,p2,bpp); \
	JASDL_PutRawPixel(ptr2,p,bpp); \
}
	if(rf&JASDL_RF_XY)
	{
		for(y=0; y<s->h; y++)
			for(ptr=s->pixels+y*w, ptr2=s->pixels+y*bpp;
					ptr>ptr2;
					ptr+=bpp, ptr2+=w)
				swap;
	}
	if(rf&JASDL_RF_X)
	{
		for(y=0; y<s->h; y++)
			for(ptr=s->pixels+y*w, ptr2=ptr+(s->w-1)*bpp;
					ptr<ptr2;
					ptr+=bpp,ptr2-=bpp)
				swap;
	}
	if(rf&JASDL_RF_Y)
	{
		for(x=0; x<s->w; x++)
			for(ptr=s->pixels+x*bpp, ptr2=ptr+(s->h-1)*w;
					ptr<ptr2;
					ptr+=w, ptr2-=w)
				swap;
	}
#undef swap 
	JASDL_SafeUnlockSurface(s);
}
