#ifndef jasdl_h
#define jasdl_h

#include <stdarg.h>
#include <stdio.h>

#include "SDL.h"
#include "SDL_image.h"

/* enums */

typedef enum {
	// basic ops
	JASDL_RF_XY =1,
	JASDL_RF_X  =2,
	JASDL_RF_Y  =4,
	// macro ops
	JASDL_RF_90 =5,
	JASDL_RF_180=6,
	JASDL_RF_270=3
} JASDL_RotateFlipType;

/* types */

typedef void (*RedrawFunc)(void);
typedef int (*HandleEventFunc)(SDL_Event*);

/* data */

extern SDL_Surface *s;
extern SDL_Event e;
extern Uint8 needRedraw,done;
#ifdef DOJOY
extern Uint8 joyEvent;
extern Sint16 joyY[4],joyX[4];
extern Uint32 joyButtons[4];
#endif
extern Uint16 mouseX, mouseY;
extern Uint8  mouseButtons;
extern RedrawFunc redraw;
extern HandleEventFunc handleEvent;

/* functions */
void JASDL_Init();

void JASDL_Errorv(char *str, va_list ap);
void JASDL_Error(char *str,...);
void JASDL_Exit(char *str,...);

void JASDL_SafeLockSurface(SDL_Surface *s);
void JASDL_SafeUnlockSurface(SDL_Surface *s);

void JASDL_EventPollLoop();
void JASDL_EventWaitLoop();

int JASDL_BasicHandleEvent(SDL_Event *e);

#define JASDL_PutRawPixel8(ptr,pxl) (*(Uint8*)ptr=pxl)
#define JASDL_PutRawPixel16(ptr,pxl) (*(Uint16*)ptr=pxl)
#if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
#define JASDL_PutRawPixel24(ptr,pxl) {*(Uint8*)ptr=(pxl>>16)&0xff;ptr[1]=(pxl>>8)&0xff;ptr[2]=pxl&0xff;}
#else
#define JASDL_PutRawPixel24(ptr,pxl) {ptr[2]=(pxl>>16)&0xff;ptr[1]=(pxl>>8)&0xff;*(Uint8*)ptr=pxl&0xff;}
#endif
#define JASDL_PutRawPixel32(ptr,pxl) (*(Uint32*)ptr=pxl)
#define JASDL_PutRawPixel(ptr,pxl,bpp) { \
    switch(bpp) { \
        case 1: \
            JASDL_PutRawPixel8(ptr,pxl); \
            break; \
        case 2: \
            JASDL_PutRawPixel16(ptr,pxl); \
            break; \
        case 3: \
            JASDL_PutRawPixel24(ptr,pxl); \
            break; \
        case 4: \
            JASDL_PutRawPixel32(ptr,pxl); \
            break; \
    } }

void JASDL_PutPixel(SDL_Surface *surface, int x, int y, Uint32 P);

#define JASDL_GetRawPixel8(ptr) (*ptr)
#define JASDL_GetRawPixel16(ptr) (*(Uint16*)ptr)
#if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
#define JASDL_GetRawPixel24(ptr)  (ptr[0] << 16 | ptr[1] << 8 | ptr[2])
#else
#define JASDL_GetRawPixel24(ptr)  (ptr[2] << 16 | ptr[1] << 8 | ptr[0])
#endif
#define JASDL_GetRawPixel32(ptr) (*(Uint32*)ptr)
#define JASDL_GetRawPixel(ptr,bpp) ( \
    (bpp==1)?JASDL_GetRawPixel8(ptr):\
	(bpp==2)?JASDL_GetRawPixel16(ptr):\
	(bpp==3)?JASDL_GetRawPixel24(ptr):\
	(bpp==4)?JASDL_GetRawPixel32(ptr):\
	0)

Uint32 JASDL_GetPixel(SDL_Surface *s, int x, int y);

Uint8 *JASDL_CreateIconMask(SDL_Surface *s);

SDL_Surface *JASDL_DisplayFormatAlpha(SDL_Surface *src);
SDL_Surface *JASDL_CreateDisplaySurface(Uint32 w, Uint32 h);
SDL_Surface *JASDL_CopySurface(SDL_Surface *src, SDL_Rect *rect);

void JASDL_RotateFlip(SDL_Surface *s, JASDL_RotateFlipType rf);

#endif
